﻿using System;
using System.CodeDom;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms.VisualStyles;
using ff14bot;
using ff14bot.Behavior;
using ff14bot.Enums;
using ff14bot.Managers;
using ff14bot.Navigation;
using ff14bot.Objects;
using Kupper;
using Kupper.Helpers;
using Kupper.Settings;
using Pathfinding;
using TreeSharp;
using Action = TreeSharp.Action;

namespace Kupper.Rotations
{
    public class ArcherBard : KupperRoutine
    {
        //DEVELOPERS REPLACE GetType().Name WITH YOUR CR'S NAME.
        public override string Name
        {
            get { return "Kupper [" + GetType().Name + "]"; }
        }
		
		public override void OnInitialize()
        {
            WindowSettings = new ArcherSettings();
        }

        private ArcherSettings settings
        {
            get { return WindowSettings as ArcherSettings; }
        }

        public override float PullRange
        {
            get { return 20; }
        }

        public override ClassJobType[] Class
        {
            get { return new ClassJobType[] {ClassJobType.Archer, ClassJobType.Bard,}; }
        }

		public SpellData SkillBlocker;
		
		public override void OnPulse()
        {
		Actionmanager.CurrentActions.TryGetValue("Heavy Shot", out SkillBlocker);
        }
		
        [Behavior(BehaviorType.PreCombatBuffs)]
        public Composite CreateBasicPreCombatBuffs()
        {
            return SummonChocobo();
        }

        [Behavior(BehaviorType.Rest)]
        public Composite CreateBasicRest()
        {
            return DefaultRestBehavior(r => Core.Player.CurrentTPPercent);
        }

        [Behavior(BehaviorType.Pull)]
        public Composite CreateBasicPull()
        {
            return new PrioritySelector(
                Spell.PullCast("Heavy Shot")
                );
        }

        [Behavior(BehaviorType.Combat)]
        public Composite CreateBasicCombat()
        {
            return new PrioritySelector(ctx => Core.Player.CurrentTarget as BattleCharacter,
                new Decorator(r => BotManager.Current.IsAutonomous, new PrioritySelector(
					CommonBehaviors.MoveToLos(ctx => ctx as GameObject),
					CommonBehaviors.MoveAndStop(ctx => (ctx as GameObject).Location, ctx => 24, true, "Moving to unit")
						)
					),
				
				//Interrupts
				new Decorator(r => settings.UseInterrupt && settings.Interrupt.Contains((Core.Player.CurrentTarget as BattleCharacter).SpellCastInfo.Name), new PrioritySelector(
					Spell.Cast("Blunt Arrow")
						)
					),
					

				//Buffs
				new Decorator(r => settings.UseBuffs && Core.Player.CurrentTarget.CurrentHealth >= settings.BuffHPLimit && SkillBlocker.Cooldown.TotalMilliseconds >= 1000, new PrioritySelector(
					Spell.Cast("Internal Release", r => settings.UseInternalRelease, r => Core.Player),
					Spell.Cast("Raging Strikes", r => settings.UseRagingStrikes, r => Core.Player),
					Spell.Cast("Hawk's Eye", r => settings.UseHawksEye, r => Core.Player),
					Spell.Cast("Barrage", r => settings.UseBarrage, r => Core.Player),
					Spell.Cast("Blood for Blood", r => settings.UseBloodForBlood, r => Core.Player),
					Spell.Cast("Quelling Strikes", r => settings.UseQuellingStrikes, r => Core.Player)
						)
					),
				
				//GCDs
				new Decorator(new PrioritySelector(
					Spell.Cast("Straight Shot", r => !Core.Player.HasAura("Straight Shot")),
					Spell.Apply("Windbite", r => Core.Player.CurrentTarget.CurrentHealth >= settings.DOTLimit),
					Spell.Apply("Venomous Bite", r => Core.Player.CurrentTarget.CurrentHealth >= settings.DOTLimit),
					Spell.Cast("Straight Shot", r => Core.Player.HasAura("Straighter Shot")),
					Spell.Cast("Heavy Shot")
						)
					),
					
				//Defensives
				new Decorator(r => SkillBlocker.Cooldown.TotalMilliseconds >= 1000 || Core.Player.CurrentTP <= 50, new PrioritySelector(
					Spell.Cast("Invigorate", r => Core.Player.CurrentTPPercent <= settings.UseInvigorateTP, r => Core.Player),
					Spell.Cast("Second Wind", r => Core.Player.CurrentHealthPercent <= settings.UseSecondWindHP, r => Core.Player),
					Spell.Cast("Featherfoot", r => Core.Player.CurrentHealthPercent <= settings.UseFeatherfootHP, r => Core.Player)
						)
					),
				
				//Flaming Arrow
				
				new Decorator(r => settings.UseFlamingArrow && Actionmanager.CanCast("Flaming Arrow", Core.Player.CurrentTarget) && (SkillBlocker.Cooldown.TotalMilliseconds >= 1000 || Core.Player.CurrentTP <= 50),
                    new Action(r => Actionmanager.DoActionLocation("Flaming Arrow", Core.Target.Location))
					),
					
					
				//OffGCDs
				new Decorator(r => SkillBlocker.Cooldown.TotalMilliseconds >= 1000, new PrioritySelector(
					
					Spell.Cast("Misery's End"),
					Spell.Cast("Bloodletter"),
					Spell.Cast("Repelling Shot", r => settings.UseRepellingShot),
					Spell.Cast("Blunt Arrow", r => settings.UseBluntArrow && !settings.UseInterrupt)					
						)
					)
				

			);
        }
    }
}