using System;
using System.CodeDom;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms.VisualStyles;
using ff14bot;
using ff14bot.Behavior;
using ff14bot.Enums;
using ff14bot.Managers;
using ff14bot.Navigation;
using ff14bot.Objects;
using ff14bot.Helpers;
using Kupper;
using Kupper.Helpers;
using Kupper.Settings;
using Pathfinding;
using TreeSharp;
using Action = TreeSharp.Action;

namespace Kupper.Rotations
{
    public class Rogue : KupperRoutine
    {
        //DEVELOPERS REPLACE GetType().Name WITH YOUR CR'S NAME.
        public override string Name
        {
            get { return "Kupper [" + GetType().Name + "]"; }
        }
		
		public override void OnInitialize()
        {
            WindowSettings = new RogueSettings();
        }

        private RogueSettings settings
        {
            get { return WindowSettings as RogueSettings; }
        }

        public override float PullRange
        {
            get { return 3; }
        }

        public override ClassJobType[] Class
        {
            get { return new ClassJobType[] {ClassJobType.Rogue}; }
        }

		public SpellData SkillBlocker;
		public SpellData SATATimer;
		public SpellData NinjutsuTimer;
		
		public override void OnPulse()
        {
			if(	
			Actionmanager.LastSpell.Name == "Mutilate" ||
			Actionmanager.LastSpell.Name == "Spinning Edge" ||
			Actionmanager.LastSpell.Name == "Gust Slash" ||
			Actionmanager.LastSpell.Name == "Shadow Fang" ||
			Actionmanager.LastSpell.Name == "Dancing Edge" ||
			Actionmanager.LastSpell.Name == "Aeolian Edge" ||
			Actionmanager.LastSpell.Name == "EmptySpell")
			Kupper.Settings.RogueSettings.ComboState = Actionmanager.LastSpell.Name;
			
			if(
			Actionmanager.LastSpell.Name == "Jin" ||
			Actionmanager.LastSpell.Name == "Chi" ||
			Actionmanager.LastSpell.Name == "Ten" ||
			Actionmanager.LastSpell.Name == "Ninjutsu")
			Kupper.Settings.RogueSettings.MudraState = Actionmanager.LastSpell.Name;
			
		Actionmanager.CurrentActions.TryGetValue("Spinning Edge", out SkillBlocker);
		Actionmanager.CurrentActions.TryGetValue("Sneak Attack", out SATATimer);
		Actionmanager.CurrentActions.TryGetValue("Ninjutsu", out NinjutsuTimer);
		
        }
		
        [Behavior(BehaviorType.PreCombatBuffs)]
        public Composite CreateBasicPreCombatBuffs()
        {
            return SummonChocobo();
        }

        [Behavior(BehaviorType.Rest)]
        public Composite CreateBasicRest()
        {
            return DefaultRestBehavior(r => Core.Player.CurrentTPPercent);
        }

        [Behavior(BehaviorType.Pull)]
        public Composite CreateBasicPull()
        {
            return new PrioritySelector(
				//SATA
				new Decorator(r =>
				
					Core.Player.HasAura("Hidden"),
				
					new PrioritySelector(
						Spell.Cast("Trick Attack", r => Core.Player.CurrentTarget.IsBehind && settings.UseTrickAttack),
						Spell.Cast("Sneak Attack", r => !Core.Player.CurrentTarget.IsBehind && !Core.Player.CurrentTarget.IsFlanking && settings.UseSneakAttack)
						)
					),
						
				//Buffs
				new Decorator(r => 
				
					settings.UseBuffs && 
					Core.Player.CurrentTarget.CurrentHealth >= settings.BuffHPLimit && 
					SkillBlocker.Cooldown.TotalMilliseconds >= 1000, 
					
					new PrioritySelector(
						Spell.Cast("Kiss of the Wasp", r => !Core.Player.HasAura("Kiss of the Wasp") && settings.UseWasp, r => Core.Player),
						Spell.Cast("Blood for Blood", r => !Core.Player.HasAura("Blood for Blood") && settings.UseBloodForBlood, r => Core.Player),
						Spell.Cast("Internal Release", r => !Core.Player.HasAura("Internal Release") && settings.UseInternalRelease, r => Core.Player)
						)
					),

				//GCDs / Main Combo
				new Decorator(				
					new PrioritySelector(
						Spell.Cast("Mutilate", r => 
							(!Core.Player.CurrentTarget.HasAura("Mutilation", true, 2000) && Core.Player.CurrentTarget.CurrentHealth >= settings.DOTLimit) && 
								(
								(Kupper.Settings.RogueSettings.ComboState == "Spinning Edge" && !Actionmanager.HasSpell("Gust Slash")) ||
								(Kupper.Settings.RogueSettings.ComboState == "Gust Slash" && !Actionmanager.HasSpell("Aeolian Edge")) ||
								(Kupper.Settings.RogueSettings.ComboState == "Shadow Fang") ||
								(Kupper.Settings.RogueSettings.ComboState == "Aeolian Edge") ||
								(Kupper.Settings.RogueSettings.ComboState == "Dancing Edge") ||
								(Kupper.Settings.RogueSettings.ComboState == "Mutilate") ||
								(Kupper.Settings.RogueSettings.ComboState == "EmptySpell")
								)),							
						Spell.Cast("Dancing Edge", r => 
							Kupper.Settings.RogueSettings.ComboState == "Gust Slash" && 
							(!Core.Player.CurrentTarget.HasAura("Dancing Edge", false, 2000) &&	!Core.Player.CurrentTarget.HasAura("Storm's Eye", false)) && 
							settings.UseDancingEdge),
						Spell.Cast("Aeolian Edge", r => Kupper.Settings.RogueSettings.ComboState == "Gust Slash"),
						Spell.Cast("Shadow Fang", r => Kupper.Settings.RogueSettings.ComboState == "Spinning Edge" && !Core.Player.CurrentTarget.HasAura("Shadow Fang", true, 3000) && Core.Player.CurrentTarget.CurrentHealth >= settings.DOTLimit),
						Spell.Cast("Gust Slash", r => Kupper.Settings.RogueSettings.ComboState == "Spinning Edge"),
						Spell.Cast("Spinning Edge")
						)
					),
					
				//Defensives
				new Decorator(r => 
				
					(SkillBlocker.Cooldown.TotalMilliseconds >= 1000 ||	Core.Player.CurrentTP <= 50), 
					
					new PrioritySelector(
						Spell.Cast("Invigorate", r => Core.Player.CurrentTPPercent <= settings.UseInvigorateTP, r => Core.Player),
						Spell.Cast("Second Wind", r => Core.Player.CurrentHealthPercent <= settings.UseSecondWindHP, r => Core.Player),
						Spell.Cast("Featherfoot", r => Core.Player.CurrentHealthPercent <= settings.UseFeatherfootHP, r => Core.Player)
						)
					),
				
				//OffGCDs
				new Decorator(r => 
				
					SkillBlocker.Cooldown.TotalMilliseconds >= 1000, 
					
					new PrioritySelector(
						Spell.Cast("Assassinate"),
						Spell.Cast("Mug"),
						Spell.Cast("Jugulate")
						)
					)
				);
        }

        [Behavior(BehaviorType.Combat)]
        public Composite CreateBasicCombat()
        {
            return new PrioritySelector(ctx => Core.Player.CurrentTarget as BattleCharacter,
                new Decorator(r => BotManager.Current.IsAutonomous, new PrioritySelector(
					CommonBehaviors.MoveToLos(ctx => ctx as GameObject),
					CommonBehaviors.MoveAndStop(ctx => (ctx as GameObject).Location, ctx => 24, true, "Moving to unit")
						)
					),
					
				//SATA
				new Decorator(r =>
				
					Core.Player.HasAura("Hidden"),
				
					new PrioritySelector(
						Spell.Cast("Trick Attack", r => Core.Player.CurrentTarget.IsBehind && settings.UseTrickAttack),
						Spell.Cast("Sneak Attack", r => !Core.Player.CurrentTarget.IsBehind && !Core.Player.CurrentTarget.IsFlanking && settings.UseSneakAttack)
						)
					),
						
				//Buffs
				new Decorator(r => 
				
					settings.UseBuffs && 
					Core.Player.CurrentTarget.CurrentHealth >= settings.BuffHPLimit && 
					SkillBlocker.Cooldown.TotalMilliseconds >= 1000, 
					
					new PrioritySelector(
						Spell.Cast("Kiss of the Wasp", r => !Core.Player.HasAura("Kiss of the Wasp") && settings.UseWasp, r => Core.Player),
						Spell.Cast("Blood for Blood", r => !Core.Player.HasAura("Blood for Blood") && settings.UseBloodForBlood, r => Core.Player),
						Spell.Cast("Internal Release", r => !Core.Player.HasAura("Internal Release") && settings.UseInternalRelease, r => Core.Player)
						)
					),

				//GCDs / Main Combo
				new Decorator(				
					new PrioritySelector(
						Spell.Cast("Mutilate", r => 
							(!Core.Player.CurrentTarget.HasAura("Mutilation", true, 2000) && Core.Player.CurrentTarget.CurrentHealth >= settings.DOTLimit) && 
								(
								(Kupper.Settings.RogueSettings.ComboState == "Spinning Edge" && !Actionmanager.HasSpell("Gust Slash")) ||
								(Kupper.Settings.RogueSettings.ComboState == "Gust Slash" && !Actionmanager.HasSpell("Aeolian Edge")) ||
								(Kupper.Settings.RogueSettings.ComboState == "Shadow Fang") ||
								(Kupper.Settings.RogueSettings.ComboState == "Aeolian Edge") ||
								(Kupper.Settings.RogueSettings.ComboState == "Dancing Edge") ||
								(Kupper.Settings.RogueSettings.ComboState == "Mutilate") ||
								(Kupper.Settings.RogueSettings.ComboState == "EmptySpell")
								)),							
						Spell.Cast("Dancing Edge", r => 
							Kupper.Settings.RogueSettings.ComboState == "Gust Slash" && 
							(!Core.Player.CurrentTarget.HasAura("Dancing Edge", false, 2000) &&	!Core.Player.CurrentTarget.HasAura("Storm's Eye", false)) && 
							settings.UseDancingEdge),
						Spell.Cast("Aeolian Edge", r => Kupper.Settings.RogueSettings.ComboState == "Gust Slash"),
						Spell.Cast("Shadow Fang", r => Kupper.Settings.RogueSettings.ComboState == "Spinning Edge" && !Core.Player.CurrentTarget.HasAura("Shadow Fang", true, 3000) && Core.Player.CurrentTarget.CurrentHealth >= settings.DOTLimit),
						Spell.Cast("Gust Slash", r => Kupper.Settings.RogueSettings.ComboState == "Spinning Edge"),
						Spell.Cast("Spinning Edge")
						)
					),
					
				//Defensives
				new Decorator(r => 
				
					(SkillBlocker.Cooldown.TotalMilliseconds >= 1000 ||	Core.Player.CurrentTP <= 50), 
					
					new PrioritySelector(
						Spell.Cast("Invigorate", r => Core.Player.CurrentTPPercent <= settings.UseInvigorateTP, r => Core.Player),
						Spell.Cast("Second Wind", r => Core.Player.CurrentHealthPercent <= settings.UseSecondWindHP, r => Core.Player),
						Spell.Cast("Featherfoot", r => Core.Player.CurrentHealthPercent <= settings.UseFeatherfootHP, r => Core.Player)
						)
					),
				
				//OffGCDs
				new Decorator(r => 
				
					SkillBlocker.Cooldown.TotalMilliseconds >= 1000, 
					
					new PrioritySelector(
						Spell.Cast("Assassinate"),
						Spell.Cast("Mug"),
						Spell.Cast("Jugulate")
						)
					)

			);
        }
    }
}