using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Runtime.Remoting.Messaging;
using System.Text;
using System.Windows.Forms;
using Kupper;
using Kupper.Settings;
using ff14bot;
using ff14bot.Behavior;
using ff14bot.Enums;
using ff14bot.Helpers;
using ff14bot.Managers;
using ff14bot.Navigation;
using ff14bot.Objects;
using TreeSharp;
using Action = TreeSharp.Action;
using System.ComponentModel;
using System.Globalization;
using System.Configuration;

namespace Kupper.Settings
{
    internal class ArcherSettings : KupperSettings
    {

        public class CsvConverter : TypeConverter
        {
            // Overrides the ConvertTo method of TypeConverter.
            public override object ConvertTo(ITypeDescriptorContext context,
               CultureInfo culture, object value, Type destinationType)
            {
                List<String> v = value as List<String>;
                if (destinationType == typeof(string))
                {
                    return String.Join(",", v.ToArray());
                }
                return base.ConvertTo(context, culture, value, destinationType);
            }
        }
		
        public ArcherSettings(string filename = "Archer-KupperSettings"): base(filename)
        {
        }

		[Setting]
		[Category("Combat")]
        [DefaultValue(2000)]
		[Description("DOTs will only be applied if the target's HP is above this value")]
		[DisplayName("* DOT HP Limit *")]
        public int DOTLimit { get; set; }
		
		[Setting]
		[Category("Combat")]
        [DefaultValue(true)]
		[DisplayName("Flaming Arrow")]
        public bool UseFlamingArrow { get; set; }
		
		[Setting]
		[Category("Combat")]
        [DefaultValue(true)]
		[DisplayName("Repelling Shot")]
        public bool UseRepellingShot { get; set; }
		
		[Setting]
		[Category("Combat")]
        [DefaultValue(true)]
		[DisplayName("Blunt Arrow")]
        public bool UseBluntArrow { get; set; }
		
		
		//Buffs
        [Setting]
        [Category("Buffs")]
        [DefaultValue(true)]
        [DisplayName("* Use Buffs *")]
        public bool UseBuffs { get; set; }
		
		[Setting]
        [Category("Buffs")]
        [DefaultValue(3000)]
        [Description("Buffs will only be used if the target's HP is above this value")]
		[DisplayName("* Buff HP Limit *")]
        public int BuffHPLimit { get; set; }

        [Setting]
        [DefaultValue(true)]
        [Category("Buffs")]
        [DisplayName("Internal Release")]
        public bool UseInternalRelease { get; set; }

        [Setting]
        [DefaultValue(true)]
        [Category("Buffs")]
        [DisplayName("Raging Strikes")]
        public bool UseRagingStrikes { get; set; }

        [Setting]
        [DefaultValue(true)]
        [Category("Buffs")]
        [DisplayName("Hawk's Eye")]
        public bool UseHawksEye { get; set; }

        [Setting]
        [DefaultValue(true)]
        [Category("Buffs")]
        [DisplayName("Barrage")]
        public bool UseBarrage { get; set; }
        
		[Setting]
        [DefaultValue(true)]
        [Category("Buffs")]
        [DisplayName("Quelling Strikes")]
        public bool UseQuellingStrikes { get; set; }
        
		[Setting]
        [DefaultValue(true)]
        [Category("Buffs")]
        [DisplayName("Blood for Blood")]
        public bool UseBloodForBlood { get; set; }

		//Defensives
		
		[Setting]
        [DefaultValue(54)]
        [Category("Defensives")]
        [DisplayName("Invigorate TP %")]
        public int UseInvigorateTP { get; set; }
		
		[Setting]
        [DefaultValue(40)]
        [Category("Defensives")]
        [DisplayName("Second Wind HP %")]
        public int UseSecondWindHP { get; set; }
		
		[Setting]
        [DefaultValue(40)]
        [Category("Defensives")]
        [DisplayName("Featherfoot HP %")]
        public int UseFeatherfootHP { get; set; }
		
		private List<String> _interrupt;
				
		[Description("Spells which should be interrupted.")]
        [Editor(@"System.Windows.Forms.Design.StringCollectionEditor," +
            "System.Design, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a",
           typeof(System.Drawing.Design.UITypeEditor))]
		[TypeConverter(typeof(CsvConverter))]
        [Category("Abilities")]
        [DisplayName("Spells to Interrupt")]
        public List<String> Interrupt
        {
            get
            {
                if (_interrupt == null)
                    _interrupt = new List<String>();
                return _interrupt;
            }
        }
		
		[Setting]
        [DefaultValue(false)]
        [Category("Abilities")]
        [DisplayName("Silence")]
        public bool UseInterrupt { get; set; }
		
    }

    internal class BardSettings : ArcherSettings
    {
        public BardSettings(): base("Bard-KupperSettings")
        {
        }
    }

}