using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Runtime.Remoting.Messaging;
using System.Text;
using System.Windows.Forms;
using Kupper;
using Kupper.Settings;
using ff14bot;
using ff14bot.Behavior;
using ff14bot.Enums;
using ff14bot.Helpers;
using ff14bot.Managers;
using ff14bot.Navigation;
using ff14bot.Objects;
using TreeSharp;
using Action = TreeSharp.Action;
using System.ComponentModel;
using System.Globalization;
using System.Configuration;

namespace Kupper.Settings
{
    internal class NinjaSettings : KupperSettings
    {

        public class CsvConverter : TypeConverter
        {
            // Overrides the ConvertTo method of TypeConverter.
            public override object ConvertTo(ITypeDescriptorContext context,
               CultureInfo culture, object value, Type destinationType)
            {
                List<String> v = value as List<String>;
                if (destinationType == typeof(string))
                {
                    return String.Join(",", v.ToArray());
                }
                return base.ConvertTo(context, culture, value, destinationType);
            }
        }
		
        public NinjaSettings(string filename = "Ninja-KupperSettings"): base(filename)
        {
        }
		
		[Setting]
		public static string ComboState { get; set; }
		
		[Setting]
		public static string MudraState { get; set; }

		[Setting]
		public static string lastspellused { get; set; }
		
		[Setting]
		[Category("Combat")]
        [DefaultValue(2000)]
		[Description("DOTs will only be applied if the target's HP is above this value")]
		[DisplayName("* DOT HP Limit *")]
        public int DOTLimit { get; set; }
		
		[Setting]
        [Category("Combat")]
        [DefaultValue(false)]
        [DisplayName("Sneak Attack")]
        public bool UseSneakAttack { get; set; }
		
		[Setting]
        [Category("Combat")]
        [DefaultValue(true)]
        [DisplayName("Trick	Attack")]
        public bool UseTrickAttack { get; set; }
		
		[Setting]
        [Category("Combat")]
        [DefaultValue(true)]
        [DisplayName("Dancing Edge")]
        public bool UseDancingEdge { get; set; }

		//Buffs
        [Setting]
        [Category("Buffs")]
        [DefaultValue(true)]
        [DisplayName("* Use Buffs *")]
        public bool UseBuffs { get; set; }
		
		[Setting]
        [Category("Buffs")]
        [DefaultValue(3000)]
        [Description("Buffs will only be used if the target's HP is above this value")]
		[DisplayName("* Buff HP Limit *")]
        public int BuffHPLimit { get; set; }

        [Setting]
        [DefaultValue(true)]
        [Category("Buffs")]
        [DisplayName("Internal Release")]
        public bool UseInternalRelease { get; set; }

		[Setting]
        [DefaultValue(true)]
        [Category("Buffs")]
        [DisplayName("Blood for Blood")]
        public bool UseBloodForBlood { get; set; }
		
		[Setting]
        [DefaultValue(true)]
        [Category("Buffs")]
        [DisplayName("Kiss of the Wasp")]
        public bool UseWasp { get; set; }
		
		[Setting]
        [DefaultValue(true)]
        [Category("Buffs")]
        [DisplayName("Kiss of the Viper")]
        public bool UseViper { get; set; }
		
		//Defensives
		
		[Setting]
        [DefaultValue(54)]
        [Category("Defensives")]
        [DisplayName("Invigorate TP %")]
        public int UseInvigorateTP { get; set; }
		
		[Setting]
        [DefaultValue(40)]
        [Category("Defensives")]
        [DisplayName("Second Wind HP %")]
        public int UseSecondWindHP { get; set; }
		
		[Setting]
        [DefaultValue(40)]
        [Category("Defensives")]
        [DisplayName("Featherfoot HP %")]
        public int UseFeatherfootHP { get; set; }
		
		//Ninjutsu
		
		[Setting]
        [DefaultValue(true)]
        [Category("Ninjutsu")]
        [DisplayName("Kassatsu")]
        public bool UseKassatsu { get; set; }
		
		[Setting]
        [DefaultValue(true)]
        [Category("Ninjutsu")]
        [DisplayName("Huton")]
        public bool UseHuton { get; set; }
		
		[Setting]
        [DefaultValue(true)]
        [Category("Ninjutsu")]
        [DisplayName("Suiton")]
        public bool UseSuiton { get; set; }
		
		[Setting]
        [DefaultValue(true)]
        [Category("Ninjutsu")]
        [DisplayName("Raiton")]
        public bool UseRaiton { get; set; }
		
		[Setting]
        [DefaultValue(true)]
        [Category("Ninjutsu")]
        [DisplayName("Fuma Shuriken")]
        public bool UseFumaShuriken { get; set; }
		
    }


}